#pragma once

#include "Critical.h"
#include "Array.h"
#include "Vec3.h"

namespace Framework
{
    class Zeichnung3D; // Zeichnung.h
    class Render3D; // Render3D.h
    struct MausEreignis3D; // MausEreignis.h
    class Model3D;
    struct DiffuseLight;
    struct PointLight;
    class DXBuffer;

    // Speichert alle 3D Zeichnungen einer Szene ab
    class Welt3D
    {
    protected:
        DiffuseLight *diffuseLights;
        int diffuseLightCount;
        PointLight *pointLights;
        int pointLightCount;

    private:
        RCArray< Model3D > *members;
        bool rend;
        Critical cs;
        int ref;

    public:
        // Konstructor
        __declspec( dllexport ) Welt3D();
        // Destruktor
        __declspec( dllexport ) virtual ~Welt3D();
        // Blockiert den zugriff auf das Objekt und wartet gegebenfalls auf den Zugriff
        __declspec( dllexport ) void lock();
        // Gibt das Objekt fr andere Threads frei
        __declspec( dllexport ) void unlock();
        // Fgt der Welt ein Objekt hinzu
        //  obj: Das Objekt, was hinzugefgt werden soll
        __declspec( dllexport ) void addZeichnung( Model3D *obj );
        // Entfernt ein Objekt aus der Welt
        //  obj: Das Objekt, das entwernt werden soll (ohne erhhten reference Counter)
        __declspec( dllexport ) void removeZeichnung( Model3D *zObj );
        // Verarbeitet ein Mausereignis
        //  me: Das Mausereignis, das verarbeitet werden soll
        __declspec( dllexport ) void doMausEreignis( MausEreignis3D &me );
        // Verarbeitet die vergangene Zeit
        //  tickval: Die zeit in sekunden, die seit dem letzten Aufruf der Funktion vergangen ist
        //  return: true, wenn sich das Objekt verndert hat, false sonnst.
        __declspec( dllexport ) virtual bool tick( double tickval );
        // brerechnet die Farbe eines Sichtstrahls, der von einem bestimmten punkt aus in eine bestimmte richtung schaut
        //  point: Der ursprung des Strahls,
        //  dir: Die Richtung des Strahls
        //  return: Die Farbe des Strahls
        __declspec( dllexport ) virtual int traceRay( Vec3< float > &point, Vec3< float > &dir );
        // Gibt einen Iterator zurck, mit dem alle Members aufgezhlt werden knnen
        __declspec( dllexport ) Iterator< Model3D * > getMembers();
        // Gibt die Anzahl an Punkt Lichtquellen zurck
        __declspec( dllexport ) int getPointLightCount() const;
        // Gibt die Anzahl an Richtungs Lichtquellen zurck
        __declspec( dllexport ) int getDiffuseLightCount() const;
        // Kopiert alle Lichtquellen in die Buffer
        //  zDiffuse: der Buffer fr die rechtungs Lichtquellen
        //  zPoints: der Buffer fr die Punkt Lichtquellen
        __declspec( dllexport ) void copyLight( DXBuffer *zDiffuse, DXBuffer *zPoints ) const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) Welt3D *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) Welt3D *release();
    };
}